/*
*/
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ESP8266WebServer.h>

const char ssid[] = "FSM";
const char password[] = "0101010101";

ESP8266WebServer server(80);

// D3 is GPIO5 in esp8266 board
#define Sensor_IN  5
#define Sensor_OUT 4

#define wrlo digitalWrite(Sensor_OUT, LOW);
#define wrhi digitalWrite(Sensor_OUT, HIGH);

void setup() 
{
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, LOW); // LED on
  Serial.begin(115200);
  Serial.println("\nstart"); 

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) 
  {
    delay(500);
    Serial.print(".");
  }
  Serial.println(WiFi.localIP());
  
  server.on("/", handleRoot);
  server.on("/test.svg", drawGraph);
  server.onNotFound(handleNotFound);
  
  server.begin();
  Serial.println("HTTP server started");

  digitalWrite(LED_BUILTIN, HIGH); // LED off
  pinMode(Sensor_IN, INPUT);
  pinMode(Sensor_OUT, OUTPUT);
  // OC version
  //pinMode(Sensor_OUT, OUTPUT_OPEN_DRAIN); 
  wrhi
}

//alternatively micros has 4us resolution
//1000 ticks per ms. unsigned long
const uint32_t tickperms = 1000;
const uint32_t tickpermshalf = tickperms/2;

uint32_t count;    // 1ms = 1000
uint32_t oldcount; 

uint32_t len; 
uint8_t sensorState,lastsensorState;   

uint8_t bitarr[8*15];
uint8_t bitarrpt = 0;
uint8_t bytearr[15];

void build()
{
  for (int k=0;k<15;k++) 
  {
    uint8_t mybyte = 0;
    for (int i=0;i<8;i++)
      mybyte |= bitarr[k*8+i]<<i;
    bytearr[k] = mybyte;
  }
}
void display()
{
  for (int i=0;i<15;i++) 
  {
    Serial.print(bytearr[i],HEX);
    Serial.print(" ");
  }
  Serial.println();  
}

uint8_t tempin = 0, tempout = 0;
bool wpon =  false;

void analyze()
{
  if (bytearr[0] == 0xDD) // temp
  {
    tempin  = bytearr[1];
    tempout = bytearr[2];
    wpon = (bytearr[8] == 0x1D);
    storewp(tempin,tempout,wpon);
  } 
}

void wirtebyte(uint8_t mybyte)
{
  for (uint8_t k=0;k<8;k++)
  {
    delay(1);
    wrhi
    if (mybyte & 1) delay(1);
    else            delay(3);
    wrlo
    mybyte=mybyte>>1; 
  }
}

uint8_t wpmpon[15] = {0x33, 0xCD, 0xD7, 0xD2, 0xFC, 0xF2, 0x5F, 0x11, 0xD7, 0xCD, 0xFF, 0xF5, 0xFA, 0xDC, 0x4E };
uint8_t elecon[15] = {0x33, 0xCD, 0xD7, 0xD2, 0xFC, 0xF2, 0x5F, 0x50, 0xD7, 0xCD, 0xFF, 0xF5, 0xFA, 0xDC, 0x8D };
uint8_t alloff[15] = {0x33, 0xCD, 0xD7, 0xD2, 0xFC, 0xF2, 0x5F, 0x51, 0xD7, 0xCD, 0xFF, 0xF5, 0xFA, 0xDC, 0x8E };

void writemsg(uint8_t * msg)
{
  Serial.println("SEND");
  delay(150); //wait for in between msg's
        
  // sync 9 ms low 5ms hi, 14ms total
  wrlo
  delay(9);
  wrhi
  delay(5);
  wrlo

  // write msg
  for (int i=0;i<15;i++) wirtebyte(msg[i]);
  
  // end
  delay(1);
  wrhi
}

char sendcmd = 0;

void loop() 
{
  // do http server
  if (bitarrpt == 8*15) // stalls when no input signal
    server.handleClient();
      
  if (bitarrpt == 8*15) // rx message did end here
  {
    if (sendcmd != 0)
    {
      if (sendcmd=='o') writemsg(alloff);
      if (sendcmd=='e') writemsg(elecon);
      if (sendcmd=='w') writemsg(wpmpon);
      sendcmd = 0;   
    }
  }
      
  sensorState = digitalRead(Sensor_IN);
  if (sensorState != lastsensorState)
  {
    lastsensorState = sensorState;
    count = micros();
    len = count - oldcount;
    oldcount = count;
    //Serial.println(len); 
    
    if (sensorState == 0) // high to low transition
    {
      if      (len < tickpermshalf) Serial.println("errlo ");
      else if (len < tickperms + tickpermshalf) // was one
      {
        bitarr[bitarrpt++] = 1;
        //Serial.print("1");
      }
      else if (len < 3 * tickperms + tickpermshalf) // was zero
      {
        bitarr[bitarrpt++] = 0;
        //Serial.print("0");
      }
      else if (len < 5 * tickperms + tickpermshalf) // was 5ms sync
      {
        //Serial.println(bitarrpt);
        bitarrpt = 0;
      }
      else if (len > 100 * tickperms) // was 400ms intermessage gap 
      {
        // got 9ms time here
        if (bitarrpt != 8*15) 
        {
          Serial.print("wrong no of bits ");
          Serial.println(bitarrpt);
        }
        else
        {
          build();
          display();
          analyze();
        }
        bitarrpt = 0;
        // check if send wanted
        if (Serial.available()) sendcmd = Serial.read();
      }      
      if (bitarrpt > 8*15) 
      {
        bitarrpt = 0;
        Serial.println("too many bits");
      }
    } 
  }
}
